@@ -535,6 +535,9 @@ pub enum ConnError {
     ClosedInvalidScreen,
     /// Connection closed because some file descriptor passing operation failed.
     ClosedFdPassingFailed,
+    /// XOpenDisplay returned NULL
+    #[cfg(feature = "xlib_xcb")]
+    XOpenDisplay,
 }
 
 impl ConnError {
@@ -553,6 +556,10 @@ impl ConnError {
             ConnError::ClosedFdPassingFailed => {
                 "Connection closed, some file descriptor passing operation failed"
             }
+            #[cfg(feature = "xlib_xcb")]
+            ConnError::XOpenDisplay => {
+                "XOpenDisplay failed to open a display. Check the $DISPLAY env var"
+            }
         }
     }
 }
@@ -722,6 +729,9 @@ impl Connection {
     pub fn connect_with_xlib_display() -> ConnResult<(Connection, i32)> {
         unsafe {
             let dpy = xlib::XOpenDisplay(ptr::null());
+            if dpy.is_null() {
+                return Err(ConnError::XOpenDisplay);
+            }
 
             check_connection_error(XGetXCBConnection(dpy))?;
 
@@ -752,6 +762,9 @@ impl Connection {
     ) -> ConnResult<(Connection, i32)> {
         unsafe {
             let dpy = xlib::XOpenDisplay(ptr::null());
+            if dpy.is_null() {
+                return Err(ConnError::XOpenDisplay);
+            }
 
             check_connection_error(XGetXCBConnection(dpy))?;
 
@@ -1157,6 +1170,16 @@ impl Connection {
         }
     }
 
+    /// Resolve an xcb_generic_event_t pointer into an Event.
+    /// # Safety
+    /// The caller is repsonsible to ensure that the `ev` pointer is not NULL.
+    /// The ownership of the pointer is effectively transferred to the
+    /// returned Event and it will be destroyed when the Event is
+    /// dropped.
+    pub unsafe fn resolve_event(&self, ev: &mut xcb_generic_event_t) -> Event {
+        event::resolve_event(ev, &self.ext_data)
+    }
+
     unsafe fn handle_poll_for_event(&self, ev: *mut xcb_generic_event_t) -> Result<Option<Event>> {
         if ev.is_null() {
             self.has_error()?;
@@ -1601,6 +1624,12 @@ impl Connection {
     }
 }
 
+impl AsRef<Connection> for Connection {
+    fn as_ref(&self) -> &Connection {
+        self
+    }
+}
+
 impl AsRawFd for Connection {
     fn as_raw_fd(&self) -> RawFd {
         unsafe { xcb_get_file_descriptor(self.c) }
@@ -1646,6 +1675,7 @@ mod dan {
     pub(crate) static mut DAN_CONN: *mut xcb_connection_t = ptr::null_mut();
 
     impl fmt::Debug for x::Atom {
+        #[allow(clippy::print_in_format_impl)]
         fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
             if self.resource_id() == 0 {
                 return f.write_str("ATOM_NONE");
